#!/usr/bin/env node
import {
  startServer
} from "./chunk-6X572QBT.js";
import "./chunk-5UQ3B3EM.js";
import "./chunk-7KVK3LMR.js";
import "./chunk-Q7GF65O3.js";

// src/bin.ts
import { config } from "dotenv";
import { resolve } from "path";
config({ path: resolve(process.cwd(), ".env") });
startServer().catch((error) => {
  console.error("Failed to start server:", error);
  process.exit(1);
});
